@@ -11,7 +11,7 @@
  ********************************************************************
 
  function: random psychoacoustics (not including preecho)
- last mod: $Id: psy.h,v 1.32 2002/07/13 06:12:47 xiphmont Exp $
+ last mod: $Id: psy.h 7187 2004-07-20 07:24:27Z xiphmont $
 
  ********************************************************************/
 
@@ -109,6 +109,9 @@ typedef struct {
   int   eighth_octave_lines; /* power of two, please */
   int   total_octave_lines;  
   long  rate; /* cache it */
+
+  float m_val; /* Masking compensation value */
+
 } vorbis_look_psy;
 
 extern void   _vp_psy_init(vorbis_look_psy *p,vorbis_info_psy *vi,
@@ -139,7 +142,9 @@ extern void _vp_offset_and_mix(vorbis_look_psy *p,
 			       float *noise,
 			       float *tone,
 			       int offset_select,
-			       float *logmask);
+			       float *logmask,
+			       float *mdct,
+			       float *logmdct);
 
 extern float _vp_ampmax_decay(float amp,vorbis_dsp_state *vd);
 
@@ -171,5 +176,11 @@ extern int **_vp_quantize_couple_sort(vorbis_block *vb,
 				      vorbis_info_mapping0 *vi,
 				      float **mags);
 
+extern void hf_reduction(vorbis_info_psy_global *g,
+			 vorbis_look_psy *p,
+			 vorbis_info_mapping0 *vi,
+			 float **mdct);
+
+
 #endif
 
