@@ -11,7 +11,7 @@
  ********************************************************************
 
  function: simple programmatic interface for encoder mode setup
- last mod: $Id: vorbisenc.c,v 1.47 2002/07/11 06:40:50 xiphmont Exp $
+ last mod: $Id: vorbisenc.c 7451 2004-08-02 08:04:52Z xiphmont $
 
  ********************************************************************/
 
@@ -162,14 +162,10 @@ static vorbis_info_mapping0 _map_nominal[2]={
 
 static ve_setup_data_template *setup_list[]={
   &ve_setup_44_stereo,
-  &ve_setup_44_stereo_low,
   &ve_setup_44_uncoupled,
-  &ve_setup_44_uncoupled_low,
 
   &ve_setup_32_stereo,
-  &ve_setup_32_stereo_low,
   &ve_setup_32_uncoupled,
-  &ve_setup_32_uncoupled_low,
 
   &ve_setup_22_stereo,
   &ve_setup_22_uncoupled,
@@ -183,8 +179,6 @@ static ve_setup_data_template *setup_list[]={
 
   &ve_setup_X_stereo,
   &ve_setup_X_uncoupled,
-  &ve_setup_X_stereo_low,
-  &ve_setup_X_uncoupled_low,
   &ve_setup_XX_stereo,
   &ve_setup_XX_uncoupled,
   0
@@ -659,6 +653,7 @@ static void get_setup_template(vorbis_info *vi,
 	  float del=(req-low)/(high-low);
 	  hi->base_setting=j+del;
 	}
+
 	return;
       }
     }
@@ -829,21 +824,30 @@ int vorbis_encode_setup_init(vorbis_info *vi){
   vorbis_encode_map_n_res_setup(vi,hi->base_setting,setup->maps);
 
   /* set bitrate readonlies and management */
-  vi->bitrate_nominal=setting_to_approx_bitrate(vi);
+  if(hi->bitrate_av>0)
+    vi->bitrate_nominal=hi->bitrate_av;
+  else{
+    vi->bitrate_nominal=setting_to_approx_bitrate(vi);
+  }
+
   vi->bitrate_lower=hi->bitrate_min;
   vi->bitrate_upper=hi->bitrate_max;
-  vi->bitrate_window=hi->bitrate_limit_window;
+  if(hi->bitrate_av)
+    vi->bitrate_window=(double)hi->bitrate_reservoir/hi->bitrate_av;
+  else
+    vi->bitrate_window=0.;
 
   if(hi->managed){
-    ci->bi.queue_avg_time=hi->bitrate_av_window;
-    ci->bi.queue_avg_center=hi->bitrate_av_window_center;
-    ci->bi.queue_minmax_time=hi->bitrate_limit_window;
-    ci->bi.queue_hardmin=hi->bitrate_min;
-    ci->bi.queue_hardmax=hi->bitrate_max;
-    ci->bi.queue_avgmin=hi->bitrate_av_lo;
-    ci->bi.queue_avgmax=hi->bitrate_av_hi;
-    ci->bi.avgfloat_downslew_max=-999999.f;
-    ci->bi.avgfloat_upslew_max=999999.f;
+    ci->bi.avg_rate=hi->bitrate_av;
+    ci->bi.min_rate=hi->bitrate_min;
+    ci->bi.max_rate=hi->bitrate_max;
+
+    ci->bi.reservoir_bits=hi->bitrate_reservoir;
+    ci->bi.reservoir_bias=
+      hi->bitrate_reservoir_bias;
+    
+    ci->bi.slew_damp=hi->bitrate_av_damp;
+
   }
 
   return(0);
@@ -902,7 +906,7 @@ int vorbis_encode_setup_vbr(vorbis_info *vi,
   codec_setup_info *ci=vi->codec_setup;
   highlevel_encode_setup *hi=&ci->hi;
 
-  quality+=.00001;
+  quality+=.0000001;
   if(quality>=1.)quality=.9999;
 
   get_setup_template(vi,channels,rate,quality,0);
@@ -946,7 +950,10 @@ int vorbis_encode_setup_managed(vorbis_info *vi,
 
   if(nominal_bitrate<=0.){
     if(max_bitrate>0.){
-      nominal_bitrate=max_bitrate*.875;
+      if(min_bitrate>0.)
+	nominal_bitrate=(max_bitrate+min_bitrate)*.5;
+      else
+	nominal_bitrate=max_bitrate*.875;
     }else{
       if(min_bitrate>0.){
 	nominal_bitrate=min_bitrate;
@@ -966,15 +973,13 @@ int vorbis_encode_setup_managed(vorbis_info *vi,
   }
 
   /* initialize management with sane defaults */
-      /* initialize management with sane defaults */
   hi->managed=1;
-  hi->bitrate_av_window=4.;
-  hi->bitrate_av_window_center=.5;
-  hi->bitrate_limit_window=2.;
   hi->bitrate_min=min_bitrate;
   hi->bitrate_max=max_bitrate;
-  hi->bitrate_av_lo=tnominal;
-  hi->bitrate_av_hi=tnominal;
+  hi->bitrate_av=tnominal;
+  hi->bitrate_av_damp=1.5f; /* full range in no less than 1.5 second */
+  hi->bitrate_reservoir=nominal_bitrate*2;
+  hi->bitrate_reservoir_bias=.1; /* bias toward hoarding bits */
 
   return(ret);
 
@@ -1010,8 +1015,10 @@ int vorbis_encode_ctl(vorbis_info *vi,int number,void *arg){
     int setp=(number&0xf); /* a read request has a low nibble of 0 */
 
     if(setp && hi->set_in_stone)return(OV_EINVAL);
-
+    
     switch(number){
+      
+    /* now deprecated *****************/
     case OV_ECTL_RATEMANAGE_GET:
       {
 	
@@ -1019,17 +1026,18 @@ int vorbis_encode_ctl(vorbis_info *vi,int number,void *arg){
 	  (struct ovectl_ratemanage_arg *)arg;
 	
 	ai->management_active=hi->managed;
-	ai->bitrate_av_window=hi->bitrate_av_window;
-	ai->bitrate_av_window_center=hi->bitrate_av_window_center;
-	ai->bitrate_hard_window=hi->bitrate_limit_window;
+	ai->bitrate_hard_window=ai->bitrate_av_window=
+	  (double)hi->bitrate_reservoir/vi->rate;
+	ai->bitrate_av_window_center=1.;
 	ai->bitrate_hard_min=hi->bitrate_min;
 	ai->bitrate_hard_max=hi->bitrate_max;
-	ai->bitrate_av_lo=hi->bitrate_av_lo;
-	ai->bitrate_av_hi=hi->bitrate_av_hi;
+	ai->bitrate_av_lo=hi->bitrate_av;
+	ai->bitrate_av_hi=hi->bitrate_av;
 	
       }
       return(0);
     
+    /* now deprecated *****************/
     case OV_ECTL_RATEMANAGE_SET:
       {
 	struct ovectl_ratemanage_arg *ai=
@@ -1044,33 +1052,19 @@ int vorbis_encode_ctl(vorbis_info *vi,int number,void *arg){
       }
       return 0;
 
+    /* now deprecated *****************/
     case OV_ECTL_RATEMANAGE_AVG:
       {
 	struct ovectl_ratemanage_arg *ai=
 	  (struct ovectl_ratemanage_arg *)arg;
 	if(ai==NULL){
-	  hi->bitrate_av_lo=0;
-	  hi->bitrate_av_hi=0;
-	  hi->bitrate_av_window=0;
+	  hi->bitrate_av=0;
 	}else{
-	  hi->bitrate_av_window=ai->bitrate_av_window;
-	  hi->bitrate_av_window_center=ai->bitrate_av_window_center;
-	  hi->bitrate_av_lo=ai->bitrate_av_lo;
-	  hi->bitrate_av_hi=ai->bitrate_av_hi;
+	  hi->bitrate_av=(ai->bitrate_av_lo+ai->bitrate_av_hi)*.5;
 	}
-
-	if(hi->bitrate_av_window<.25)hi->bitrate_av_window=.25;
-	if(hi->bitrate_av_window>10.)hi->bitrate_av_window=10.;
-	if(hi->bitrate_av_window_center<0.)hi->bitrate_av_window=0.;
-	if(hi->bitrate_av_window_center>1.)hi->bitrate_av_window=1.;
-	
-	if( ( (hi->bitrate_av_lo<=0 && hi->bitrate_av_hi<=0)||
-	      (hi->bitrate_av_window<=0) ) &&
-	    ( (hi->bitrate_min<=0 && hi->bitrate_max<=0)||
-	      (hi->bitrate_limit_window<=0) ))
-	  hi->managed=0;
       }
       return(0);
+    /* now deprecated *****************/
     case OV_ECTL_RATEMANAGE_HARD:
       {
 	struct ovectl_ratemanage_arg *ai=
@@ -1078,23 +1072,79 @@ int vorbis_encode_ctl(vorbis_info *vi,int number,void *arg){
 	if(ai==NULL){
 	  hi->bitrate_min=0;
 	  hi->bitrate_max=0;
-	  hi->bitrate_limit_window=0;
 	}else{
-	  hi->bitrate_limit_window=ai->bitrate_hard_window;
 	  hi->bitrate_min=ai->bitrate_hard_min;
 	  hi->bitrate_max=ai->bitrate_hard_max;
+	  hi->bitrate_reservoir=ai->bitrate_hard_window*
+	    (hi->bitrate_max+hi->bitrate_min)*.5;
 	}
-	if(hi->bitrate_limit_window<0.)hi->bitrate_limit_window=0.;
-	if(hi->bitrate_limit_window>10.)hi->bitrate_limit_window=10.;
-	
-	if( ( (hi->bitrate_av_lo<=0 && hi->bitrate_av_hi<=0)||
-	      (hi->bitrate_av_window<=0) ) &&
-	    ( (hi->bitrate_min<=0 && hi->bitrate_max<=0)||
-	      (hi->bitrate_limit_window<=0) ))
-	  hi->managed=0;
-      }
+	if(hi->bitrate_reservoir<128.)
+	  hi->bitrate_reservoir=128.;
+      }   
       return(0);
 
+      /* replacement ratemanage interface */
+    case OV_ECTL_RATEMANAGE2_GET:
+      {
+	struct ovectl_ratemanage2_arg *ai=
+	  (struct ovectl_ratemanage2_arg *)arg;
+	if(ai==NULL)return OV_EINVAL;
+	
+	ai->management_active=hi->managed;
+	ai->bitrate_limit_min_kbps=hi->bitrate_min;
+	ai->bitrate_limit_max_kbps=hi->bitrate_max;
+	ai->bitrate_average_kbps=hi->bitrate_av;
+	ai->bitrate_average_damping=hi->bitrate_av_damp;
+	ai->bitrate_limit_reservoir_bits=hi->bitrate_reservoir;
+	ai->bitrate_limit_reservoir_bias=hi->bitrate_reservoir_bias;
+      }
+      return (0);
+    case OV_ECTL_RATEMANAGE2_SET:
+      {
+	struct ovectl_ratemanage2_arg *ai=
+	  (struct ovectl_ratemanage2_arg *)arg;
+	if(ai==NULL){
+	  hi->managed=0;
+	}else{
+	  /* sanity check; only catch invariant violations */
+	  if(ai->bitrate_limit_min_kbps>0 &&
+	     ai->bitrate_average_kbps>0 &&
+	     ai->bitrate_limit_min_kbps>ai->bitrate_average_kbps)
+	    return OV_EINVAL;
+
+	  if(ai->bitrate_limit_max_kbps>0 &&
+	     ai->bitrate_average_kbps>0 &&
+	     ai->bitrate_limit_max_kbps<ai->bitrate_average_kbps)
+	    return OV_EINVAL;
+
+	  if(ai->bitrate_limit_min_kbps>0 &&
+	     ai->bitrate_limit_max_kbps>0 &&
+	     ai->bitrate_limit_min_kbps>ai->bitrate_limit_max_kbps)
+	    return OV_EINVAL;
+
+	  if(ai->bitrate_average_damping <= 0.)
+	    return OV_EINVAL;
+
+	  if(ai->bitrate_limit_reservoir_bits < 0)
+	    return OV_EINVAL;
+
+	  if(ai->bitrate_limit_reservoir_bias < 0.)
+	    return OV_EINVAL;
+
+	  if(ai->bitrate_limit_reservoir_bias > 1.)
+	    return OV_EINVAL;
+
+	  hi->managed=ai->management_active;
+	  hi->bitrate_min=ai->bitrate_limit_min_kbps;
+	  hi->bitrate_max=ai->bitrate_limit_max_kbps;
+	  hi->bitrate_av=ai->bitrate_average_kbps;
+	  hi->bitrate_av_damp=ai->bitrate_average_damping;
+	  hi->bitrate_reservoir=ai->bitrate_limit_reservoir_bits;
+	  hi->bitrate_reservoir_bias=ai->bitrate_limit_reservoir_bias;
+	}
+      }
+      return 0;
+      
     case OV_ECTL_LOWPASS_GET:
       {
 	double *farg=(double *)arg;
